/** \file maximDeviceSpecificUtilities.c ****************************************
 * 
 *             Project: Maxim Plug-in Peripheral Modules
 *            Filename: maximDeviceSpecificUtilities.c
 *         Description: This module contains the functions which are specific
 *                      to the invidiual modules.  These low level functions 
 *                      are not specific to the example/demo programs and could
 *                      be cut/pasted into the user's application as a starting
 *                      point for development of an end application.
 *
 *    Revision History:
 *\n       7-20-12 Rev 1.4 Nathan Young    Initial release
 *\n       9-04-12 Rev 1.5 Nathan Young    Multiple PMOD port support
 * 
 *  --------------------------------------------------------------------
 * 
 *    This code follows the following naming conventions.
 * 
 * 	  char                   chPmodValue
 * 	  char (array)           sPmodString[16]
 * 	  float                  fPmodValue
 * 	  int                    nPmodValue
 * 	  int (array)            anPmodValue[16]
 * 	  u16                    uPmodValue
 * 	  u8                     uchPmodValue
 * 	  u8 (array)             auchPmodBuffer[16]
 * 	  unsigned int           unPmodValue
 * 	  int *                  punPmodValue
 * 
 * ------------------------------------------------------------------------- */
/*
 * Copyright (C) 2012 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 ***************************************************************************/

#ifndef MAXIMDEVICESPECIFICUTILITIES_H_
#define MAXIMDEVICESPECIFICUTILITIES_H_

#include "xbasic_types.h"
#include "xspi_l.h"
#include "stdio.h"
#include "maximPMOD.h"

#define DS3231M_DOW_REGISTER 0x03       //!< Primary configuration register
#define DS3231M_STATUS_REG 0x0F         //!< Status register
#define DS3231M_EN32KHZ 0x08            //!< Register to turn 32kHz output on/off

#define DS3231M_TIMEDATE 0              //!< Specify Time/Date for time setting functions
#define DS3231M_ALARM1 1                //!< Specify Alarm 1 for time setting functions
#define DS3231M_ALARM2 2                //!< Specify Alarm 2 for time setting functions

#define MAX44000_STATE_BEGIN 0          //!< LUX measurement state: Beginning state.
#define MAX44000_STATE_SET_ALSPGA 1     //!< LUX measurement state: Set ADC resolution and amplifier gain.
#define MAX44000_STATE_READ_04 2        //!< LUX measurement state: Read LUX high and overflow.
#define MAX44000_STATE_READ_05 3        //!< LUX measurement state: Read LUX low byte
#define MAX44000_STATE_INCREASE_ALSPGA 4 //!< LUX measurement state: Increase PGA setting (lower gain)
#define MAX44000_STATE_PROCESS_LUX 5    //!< LUX measurement state: Scale the LUX value per PGA setting
#define MAX44000_STATE_FAILED_RW 6      //!< LUX measurement state: Abort, I2C failure
#define MAX44000_STATE_DELAY 7          //!< LUX measurement state: Delay one state
#define MAX44000_STATE_EXIT_FAILED 8    //!< LUX measurement state: Failure, exit
#define MAX44000_STATE_EXIT_SUCCESS 9   //!< LUX measurement state: Process completed, exit

#define MAX5825_2500_EXT_REFERENCE 0    //!< Value which sets external reference
#define MAX5825_2500_MV_REFERENCE 5     //!< Value which sets 2.500V internal reference
#define MAX5825_2048_MV_REFERENCE 6     //!< Value which sets 2.048V internal reference
#define MAX5825_4096_MV_REFERENCE 7     //!< Value which sets 4.094V internal reference

#define MAX5825_DEFAULT_M_ZNOT 0x00     //!< Restore selected DAC channel(s) to M/Znot setting
#define MAX5825_DEFAULT_ZERO 0x20       //!< Restore selected DAC channel(s) to zero
#define MAX5825_DEFAULT_MID 0x40        //!< Restore selected DAC channel(s) to mid-range value
#define MAX5825_DEFAULT_FULL 0x60       //!< Restore selected DAC channel(s) to full scale value
#define MAX5825_DEFAULT_RETURN 0x80     //!< Restore selected DAC channel(s) to value in RETURN reg


#define MAX4824_CSB 0x01                //!< CSB pin driven by GPIO 0 - mask = 0x01
#define MAX4824_LVL 0x02                //!< LVL pin driven by GPIO 1 - mask = 0x02
#define MAX4824_A1 0x04                 //!< A1 pin driven by GPIO 2 - mask = 0x04
#define MAX4824_A0 0x08                 //!< A0 pin driven by GPIO 3 - mask = 0x08
#define MAX4824_A2 0x10                 //!< A2 pin driven by GPIO 4 - mask = 0x10
#define MAX4824_RESB 0x20               //!< RESB pin driven by GPIO 5 - mask = 0x20
#define MAX4824_SETB 0x40               //!< SETB pin driven by GPIO 6 - mask = 0x40
#define MAX4824_PSAVE 0x80              //!< PSAVE pin driven by GPIO 7 - mask = 0x80

#define MAX31723_TEMP_READ 0x01         //!< Read address for Temperature LSB register (MSB=0x02)
#define MAX31723_LOW_ALARM_READ 0x05    //!< Read address for Low Temperature Alarm LSB register (MSB=0x06)
#define MAX31723_HIGH_ALARM_READ 0x03   //!< Read address for High Temperature Alarm LSB register (MSB=0x04)
#define MAX31723_LOW_ALARM_WRITE 0x85   //!< Write address for Low Temperature Alarm LSB register (MSB=0x86)
#define MAX31723_HIGH_ALARM_WRITE 0x83  //!< Write address for High Temperature Alarm LSB register (MSB=0x84)


extern XGpio g_xGpioPmodPortC;

int max_MAX44000_readLux(u32 unPeripheralAddressI2C, unsigned int *unLuxReading);
int max_MAX9611_read_silicon_die_temp(u32 unPeripheralAddressI2C, int *pnSiliconDieTemperature);
int max_MAX9611_read_ADC_voltage(u32 unPeripheralAddressI2C, u8 uchBaseRegister, unsigned int *unVoltageADC);
int max_MAX9611_turn_on_2ms_updates(u32 unPeripheralAddressI2C);
int max_MAX5380_change_voltage(u32 unPeripheralAddressI2C,u8 uchRegisterSetting);
void max_MAX5487_change_wiper(u32 unPeripheralAddressSPI, u8 uchRegisterSettingA, u8 uchRegisterSettingB, u8 uchChannelA, u8 uchChannelB);
void max_GPIO_bit_bang_SPI_write(XGpio *port, unsigned int unNumberBytes, u8 *pauchBuffer);
int max_MAX11205_GPIO_SPI_read(XGpio *port, u8 uchUseCalibrationMode);
void max_MAX5216_set_output_voltage(XGpio *pmodPort, u16 value);
int max_MAX5825_initialize(u32 unPeripheralAddressI2C);
int max_MAX5825_set_internal_reference(u32 unPeripheralAddressI2C, u8 uchReferenceCommand);
int max_MAX5825_set_DAC_12_bit_value(u32 unPeripheralAddressI2C,u8 uchChannelToSet, unsigned int unRegisterSetting);
int max_MAX5825_LOAD_DAC_from_CODE_register(u32 unPeripheralAddressI2C,u8 uchChannelToSet);
int max_MAX5825_set_CODE_register(u32 unPeripheralAddressI2C,u8 uchChannelToSet, unsigned int unRegisterSetting);
int max_MAX5825_set_RETURN_register(u32 unPeripheralAddressI2C,u8 uchChannelToSet, unsigned int unRegisterSetting);
int max_MAX5825_set_default_DAC_settings(u32 unPeripheralAddressI2C,u8 uchMask, u8 uchDefaultCodeSetting);
int max_MAX5825_send_sw_clear(u32 unPeripheralAddressI2C);
int max_MAX5825_send_sw_reset(u32 unPeripheralAddressI2C);
int max_DS3231M_32kHz(u32 unPeripheralAddressI2C, u8 uchEnable);
int max_DS3231M_get_time(u32 unPeripheralAddressI2C, struct maximDateTime *t);
int max_DS3231M_set_time(u32 unPeripheralAddressI2C, struct maximDateTime *t);
int max_DS3231M_get_temp(u32 unPeripheralAddressI2C, float *fTemp);
void max_MAX4824_set_output_inactive(XGpio *port, u8 relayAddress, int nPsaveActive);
void max_MAX4824_set_output_active(XGpio *port,u8 relayAddress, int nPsaveActive);
void max_MAX4824_set_all_inactive(XGpio *port, int nPsaveActive);
void max_MAX4824_set_all_active(XGpio *port, int nPsaveActive);
void max_MAX4824_set_powersave(XGpio *port, int nPsaveActive);
int max_MAX7304_configure_PWM(u32 unPeripheralAddressI2C,u8 uchPwm1, u8 uchPwm2, u8 uchPwm3, u8 uchPwm4);
int max_MAX7304_set_output(u32 unPeripheralAddressI2C, u8 uchHighByte, u8 uchLowByte);
int max_MAX31723_get_temp(float *fTemp, u32 unPeripheralAddressSPI, u8 uchTemperatureRegister);
int max_MAX31723_set_alarm(float fTemp, u32 unPeripheralAddressSPI, u8 uchAlarmType);
int max_MAX44000_readProx(u32 unPeripheralAddressI2C, unsigned int *unProxReading);
int max_MAX44000_set_led_drive_current(u32 unPeripheralAddressI2C, u8 uchLedDriveCurrent);
int max_MAX44000_set_als_mode(u32 unPeripheralAddressI2C);
int max_MAX44000_set_prox_mode(u32 unPeripheralAddressI2C);
int max_MAX7304_enable(u32 unPeripheralAddressI2C, u8 uchValue);
int max_MAX7304_set_direction(u32 unPeripheralAddressI2C, u8 uchHighByte, u8 uchLowByte);
int max_MAX7304_get_debounce_value(u32 unPeripheralAddressI2C, u8 *uchValue);
int max_MAX7304_set_debounce_value(u32 unPeripheralAddressI2C, u8 uchValue);
int max_MAX31855_read_thermoCoupleTemp(float *fTemp, u32 unPeripheralAddressSPI);
int max_MAX31855_read_internalTemp(float *fTemp, u32 unPeripheralAddressSPI);
int max_DS1086L_setup(u32 unPeripheralAddressI2C, XGpio *port);
int max_DS1086_get_dac(u32 unPeripheralAddressI2C, unsigned int *unDacValue);
int max_DS1086_set_dac(u32 unPeripheralAddressI2C, unsigned int unDacValue);
int max_DS1086_set_prescalar(u32 unPeripheralAddressI2C, unsigned int unPreScalarValue);
int max_DS1086_get_prescalar(u32 unPeripheralAddressI2C, unsigned int *unPreScalarValue);
int max_DS1086_set_jitter(u32 unPeripheralAddressI2C, unsigned int unJitterValue);
int max_DS1086_get_jitter(u32 unPeripheralAddressI2C, unsigned int *unJitterValue);
int max_DS1086_set_offset_delta(u32 unPeripheralAddressI2C,  int nOffsetDelta);
int max_DS1086_get_offset_delta(u32 unPeripheralAddressI2C,  int *nOffsetDelta);
int max_MAX7304_get_input(u32 unPeripheralAddressI2C, u8 *puchHighByte, u8 *puchLowByte);
int max_MAX7304_get_direction(u32 unPeripheralAddressI2C, u8 *puchHighByte, u8 *puchLowByte);
int max_MAX7304_set_output(u32 unPeripheralAddressI2C, u8 uchHighByte, u8 uchLowByte);

#endif /* MAXIMDEVICESPECIFICUTILITIES_H_ */
