/** \file menu.c ***********************************************************
 * 
 *             Project: Maxim Plug-in Peripheral Modules
 *            Filename: menu.c
 *         Description: This module contains all the functions used to
 *                      generate the menus and menu options used to run the
 *                      various demos and examples for the individual modules.
 * 
 *    Revision History:
 *\n       7-20-12 Rev 1.4 Nathan Young    Initial release
 *\n       9-04-12 Rev 1.5 Nathan Young	   Multiple PMOD port support
 * 
 *  --------------------------------------------------------------------
 * 
 *    This code follows the following naming conventions.
 * 
 * 	  char                   chPmodValue
 * 	  char (array)           sPmodString[16]
 * 	  float                  fPmodValue
 * 	  int                    nPmodValue
 * 	  int (array)            anPmodValue[16]
 * 	  u16                    uPmodValue
 * 	  u8                     uchPmodValue
 * 	  u8 (array)             auchPmodBuffer[16]
 * 	  unsigned int           unPmodValue
 * 	  int *                  punPmodValue
 * 
 * ------------------------------------------------------------------------- */
/*
 * Copyright (C) 2012 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 ***************************************************************************/

#include <stdio.h>
#include "platform.h"
#include "xgpio.h"
#include "xgpio_l.h"
#include "xparameters.h"
#include "xuartlite.h"
#include "xspi_l.h"
#include "xspi.h"
#include "xiic_l.h"
#include "utilities.h"
#include "string.h"
#include "maximPMOD.h"
#include "maximDeviceSpecificUtilities.h"
#include "menu.h"

void menu_cls()
/**
* \brief       Function to clear the screen via Hyperterminal
*
* \param       None
*
* \retval      None
*/
{
	// The following code will cause a clear screen event on Hyperterminal and many other terminal emulators
	printf("\033[2J");
}


void menu_print_maxim_banner()
/**
* \brief       Print standard Maxim banner at top of Hyperterminal screen
*
* \param       None
*
* \retval      None
*/
{
	printf( "///////////////////////////////////////////////////////////////////\r\n");
	printf( "//         ##     ##     ##     #  #  #######  ##     ##         //\r\n");
	printf( "//         # #   # #    #  #     ##      #     # #   # #         //\r\n");
	printf( "//         #  # #  #   ######    ##      #     #  # #  #         //\r\n");
	printf( "//         #   #   #  #      #  #  #  #######  #   #   #         //\r\n");
	printf( "///////////////////////////////////////////////////////////////////\r\n");
	printf("\r\n");
}


void menu_print_maxim_banner_big()
/**
* \brief       Print large Maxim banner at top of Hyperterminal screen
*
* \param       None
*
* \retval      None
*/
{
	printf( "///////////////////////////////////////////////////////////////////\r\n");
	printf( "///////////////////////////////////////////////////////////////////\r\n");
	printf( "//                                                               //\r\n");
	printf( "//         ##     ##     ##     #  #  #######  ##     ##         //\r\n");
	printf( "//         # #   # #    #  #     ##      #     # #   # #         //\r\n");
	printf( "//         #  # #  #   ######    ##      #     #  # #  #         //\r\n");
	printf( "//         #   #   #  #      #  #  #  #######  #   #   #         //\r\n");
	printf( "//                                                               //\r\n");
	printf( "//           I N T E G R A T E D    P R O D U C T S              //\r\n");
	printf( "//                     INNOVATION DELIVERED                      //\r\n");
	printf( "//                                                               //\r\n");
	printf( "///////////////////////////////////////////////////////////////////\r\n");
	printf( "///////////////////////////////////////////////////////////////////\r\n");

	printf("\r\n\r\n");
}

void menu_print_prompt()
/**
* \brief       Print a standard prompt for keyboard input "  > "
*
* \param       None
*
* \retval      None
*/
{
	printf("\r\n>> ");
	fflush(stdout);
}


void menu_print_line()
/**
* \brief       Print one line of dashes across the screen via Hyperterminal
*
* \param       None
*
* \retval      None
*/
{
	printf("----------------------------------------------------\r\n\r\n");
}

unsigned int menu_get_direct_entry(u32 nUartAddress, int nNumberBits, int nOffset)
/**
* \brief       Retrieve keyboard entry of a value via the Hyperterminal connected UART
* \par         Details
*              In most cases, this function is used to directly populate a register with a value.
*              int nNumberBits equals the number of bits in the register.  The value input by the user
*              is capped to the maximum allowable for the given number of bits (e.g. - 6 bits => '64' max)
*
* \param[in]   nUartAddress      - address of the UART peripheral in MicroBlaze memory map
* \param[in]   nNumberBits       - number of bits in register populate
* \param[in]   nOffset           - @help
*
* \retval      Value entered
*/
{
	int nTemp;
	int i;
	u8 uchInput=0;
	int nNumberCharactersReceived=0;
	u8 auchInput[8];
	unsigned int nReturnVal=0;

	menu_cls();
	nTemp = number_raised_to_power(2,nNumberBits) - 1 + nOffset;
	printf("Enter a value and then press enter (value = %d..%d)\r\n",nOffset, nTemp);
	menu_print_prompt();
	fflush(stdout);

	while(uchInput!=13)
	{
		uchInput = XUartLite_RecvByte(nUartAddress);
		if(uchInput>=48 && uchInput <=57) // digits 0..9
		{
			uchInput -=48; // subtract 48 to get the real number converted from the ASCII digit
			auchInput[nNumberCharactersReceived] = uchInput;
			nNumberCharactersReceived++;
			printf("%d",uchInput);
			fflush(stdout);
		}
		else if(uchInput==13) // carriage return
		{
			for(i=0;i<nNumberCharactersReceived;i++)
			{
				nReturnVal += auchInput[i] * number_raised_to_power(10,nNumberCharactersReceived-i-1);
			}
			if(nReturnVal > nTemp)
				nReturnVal=nTemp;
		}
	}
	printf("\r\nValue is %d\r\n",nReturnVal);
	delay(ABOUT_ONE_SECOND * 2);
	return(nReturnVal);
}

u8 menu_retrieve_keypress(u32 nUartAddress)
/**
* \brief       Get a single keypress via Hyperterminal.
* \par         Details
*              Returns ascii character corresponding to keypress with some preprocessing.
* \n           Escape sequences (Arrow keys and END) are mapped to decimal 240-244 (see defines)
* \n           Characters "0"-"9" converted to numbers 0-9
* \n           Lower case "a"-"z" converted to uppercase "A"-"Z"
*
* \param[in]   nUartAddress        - address of the UART peripheral in MicroBlaze memory map
*
* \retval      Character, partially decoded.
*/
{
	u8 uchInput;
	uchInput = XUartLite_RecvByte(nUartAddress);
	if(uchInput==27)  // Escape sequence (likely an arrow key)
	{
		uchInput = XUartLite_RecvByte(nUartAddress);
		if(uchInput==91)  // Left bracket (part #2 of the 3 part escape sequence)
		{
			uchInput = XUartLite_RecvByte(nUartAddress);
			if(uchInput==65)
				uchInput = KEYPRESS_ARROW_UP-10;
			if(uchInput==66)
				uchInput = KEYPRESS_ARROW_DOWN-10;
			if(uchInput==67)
				uchInput = KEYPRESS_ARROW_RIGHT-10;
			if(uchInput==68)
				uchInput = KEYPRESS_ARROW_LEFT-10;
			if(uchInput==75)
				uchInput = KEYPRESS_END - 10;
		}
	}
	else if(uchInput>=48 && uchInput <=57) // digits 0..9
		uchInput -=48; // subtract 48 to get the real number converted from the ASCII digit
	else if(uchInput>=97 && uchInput<=122)
		uchInput -=32;  // convert lowercase to uppercase

	return(uchInput);
}

void menu_print_main_menu(struct maximProduct *p)
/**
* \brief       Print the main menu listing choice of module to test
* \par         Details
*              This function draws data from an array structure which is populated by the
*              "menu_load_supported_maxim_products" function.
*
* \param[in]   *p        - pointer to structure containing information on each module
*
* \retval      None
*/
{
	int i;
	menu_print_maxim_banner();

	printf("Active PMOD Port = ");
	switch(g_nActivePMODPort)
	{
		case 0:
			printf("A\r\n");
			break;
		case 1:
			printf("B\r\n");
			break;
		case 2:
			printf("C\r\n");
			break;
		case 3:
			printf("D\r\n");
			break;
		default:
			printf("Error.  Invalid active port number\r\n");
			break;
	}
	printf("Press a key/number to test a Peripheral Module:\r\n");

	for(i=0i;i<NUMBER_SUPPORTED_PERIPHERAL_MODULES;i++)
	{
		printf("{%c} %s %s\r\n",p[i].uchKeypress,p[i].sPartNumber,p[i].sPartName);
	}
	menu_print_prompt();
}

void menu_load_supported_maxim_products(struct maximProduct *p)
/**
* \brief       Load the products array with data for use by "menu_print_main_menu"
* \par         Details
*              The products array is used to generate the main menu listing (of all Maxim components) and
*              is also used to associate ASCII menu keypresses with a specific function/sub-menu for that component
*
* \param[in]   *p        - pointer to structure containing information on each module
*
* \retval      None
*/
{
	int i=0;

	strcpy(p[i].sPartNumber,"DS1086L  ");
	strcpy(p[i].sPartName,"Spread Spectrum Econ Oscillator");
	p[i].uchKeypress = MENU_DS1086L;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;


	strcpy(p[i].sPartNumber,"DS3231M  ");
	strcpy(p[i].sPartName,"I2C Real-Time Clock");
	p[i].uchKeypress = MENU_DS3231M;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

	strcpy(p[i].sPartNumber,"MAX3232  ");
	strcpy(p[i].sPartName,"RS-232 Transceiver");
	p[i].uchKeypress = MENU_MAX3232;
	p[i].uchPmodType = PMOD_TYPE_UART;
	i++;

	strcpy(p[i].sPartNumber,"MAX4824  ");
	strcpy(p[i].sPartName,"Octal Relay Driver");
	p[i].uchKeypress = MENU_MAX4824;
	p[i].uchPmodType = PMOD_TYPE_GPIO;
	i++;

	strcpy(p[i].sPartNumber,"MAX5216  ");
	strcpy(p[i].sPartName,"SPI-Compatible, High performance 16 bit DAC");
	p[i].uchKeypress = MENU_MAX5216;
	p[i].uchPmodType = PMOD_TYPE_GPIO;
	i++;

	strcpy(p[i].sPartNumber,"MAX5487  ");
	strcpy(p[i].sPartName,"Dual, 256-tap SPI Digital Potentiometer");
	p[i].uchKeypress = MENU_MAX5487;
	p[i].uchPmodType = PMOD_TYPE_SPI;
	i++;

	strcpy(p[i].sPartNumber,"MAX5825  ");
	strcpy(p[i].sPartName,"I2C Octal DAC");
	p[i].uchKeypress = MENU_MAX5825;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

	strcpy(p[i].sPartNumber,"MAX7304  ");
	strcpy(p[i].sPartName,"I2C-Interfaced 16-Port, GPIO and LED Driver");
	p[i].uchKeypress = MENU_MAX7304;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

	strcpy(p[i].sPartNumber,"MAX9611  " );
	strcpy(p[i].sPartName,"Current sense amplifier with OpAmp and ADC");
	p[i].uchKeypress = MENU_MAX9611;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

	strcpy(p[i].sPartNumber,"MAX11205 ");
	strcpy(p[i].sPartName,"16 bit Delta-Sigma ADC with 2-Wire interface");
	p[i].uchKeypress = MENU_MAX11205;
	p[i].uchPmodType = PMOD_TYPE_GPIO;
	i++;

	strcpy(p[i].sPartNumber,"MAX14840 ");
	strcpy(p[i].sPartName,"RS-485 Transceiver");
	p[i].uchKeypress = MENU_MAX14840;
	p[i].uchPmodType = PMOD_TYPE_GPIO;
	i++;

	strcpy(p[i].sPartNumber,"MAX31723 ");
	strcpy(p[i].sPartName,"MAX31723 Digital Thermometer");
	p[i].uchKeypress = MENU_MAX31723;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

	strcpy(p[i].sPartNumber,"MAX31855 ");
	strcpy(p[i].sPartName,"Thermocouple to Digital Converter");
	p[i].uchKeypress = MENU_MAX31855;
	p[i].uchPmodType = PMOD_TYPE_GPIO;
	i++;

	strcpy(p[i].sPartNumber,"MAX44000 ");
	strcpy(p[i].sPartName,"I2C-Interfaced Ambient Light and Proximity Sensor");
	p[i].uchKeypress = MENU_MAX44000;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

	strcpy(p[i].sPartNumber,"PMOD     ");
	strcpy(p[i].sPartName,"Increment Active PMOD");
	p[i].uchKeypress = MENU_SET_ACTIVE_PMOD;
	p[i].uchPmodType = PMOD_TYPE_I2C;
	i++;

}

void menu_update_shadow_variable(unsigned int *panShadow,u8 uchActiveChannel, unsigned int unCodeRegisterSetting)
/**
* \brief       Support routine for MAX5825 menu to set 'shadow variable'.
* \par         Details
*              The MAX5825 menu routine keeps a local copy of the values set in the MAX5825
*              (in this application, we call this shadowing).
*              These shadow variables are used for quickly displaying the device status without
*              having to read each register in the device.  This is also convenient because the DAC register
*              values cannot be directly retrieved from the MAX5825 (as they are loaded from the CODE registers,
*              which can then later be changed).  Note that this operation is only required when displaying contents
*              of the MAX5825, as is done with the example routines.
*
* \param[in]   *panShadow	           - pointer to an unsigned integer array that 'shadows' all 8 channel registers
* \param[in]   uchActiveChannel        - value (0-7, all=8) which represents the target channel
* \param[in]   unCodeRegisterSetting   - value which will be written into the 'shadow register'
*
* \retval      None
*/
{
	unsigned int unCodeValueWritten=0;
	unsigned int *p;
	int i;

	if(unCodeRegisterSetting > 4095)
		unCodeValueWritten = 4095;
	else
		unCodeValueWritten = unCodeRegisterSetting;

	if(uchActiveChannel>=8)
	{
		p = panShadow;
		for(i=0;i<8;i++)
		{
			*p = unCodeValueWritten;
			p++;
		}
	}
	else
	{
		p = panShadow + uchActiveChannel;
		*p = unCodeValueWritten;
	}
}

void menu_copy_shadow_variable_CODE_to_DAC(unsigned int *panShadowCode, unsigned int *panShadowDac, u8 uchActiveChannel)
/**
* \brief       Support routine for MAX5825 menu to set 'shadow variable'.
* \par         Details
*              This function updates the DAC 'shadow register' with the value from the CODE 'shadow register'.
*              Note that this operation is only required when displaying contents of the MAX5825, as is done
*              with the example routines.
*
* \param[in]   *panShadowCode	    - pointer to an unsigned integer array that 'shadows' all 8 CODE channel registers
* \param[in]   *panShadowDac        - pointer to an unsigned integer array that 'shadows' all 8 DAC channel registers
* \param[in]   uchActiveChannel     - value (0-7, all=8) which represents the target channel
*
* \retval      None
*/
{
	unsigned int *pCode;
	unsigned int *pDac;
	int i;

	if(uchActiveChannel>=8)
	{
		pCode = panShadowCode;
		pDac = panShadowDac;

		for(i=0;i<8;i++)
		{
			*pDac = *pCode;
			pCode++;
			pDac++;
		}
	}
	else
	{
		pCode = panShadowCode + uchActiveChannel;
		pDac = panShadowDac + uchActiveChannel;
		*pDac = *pCode;
	}
}

void menu_MAX5825()
/**
* \brief       Menu display and processing for MAX5825
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int i=0;
	int j=0;
	unsigned int unValueDac=0;
	unsigned int unCodeRegisterSetting=0;
	unsigned int unReturnRegisterSetting=0;
	u8 uchVoltageReference2500mV=TRUE;
	u8 uchActiveChannel=8;

	unsigned int nDAC_shadow[8];
	unsigned int nCODE_shadow[8];
	unsigned int nRETURN_shadow[8];

	int menuActive=TRUE;

	// Set the PMOD Port mux so that the active port is set to I2C
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_I2C);

	for(i=0;i<8;i++)
	{
		nDAC_shadow[i]=0;
		nCODE_shadow[i]=0;
		nRETURN_shadow[i]=0;
	}


	if(check_if_i2c_device_is_present(g_nActivePMODPort))
	{
		max_MAX5825_initialize(g_unActivePeripheralAddressI2C	);		// Initialize the DAC
		delay(ABOUT_ONE_SECOND/10);
		max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C	,8,0);  // Set DACs to zero
	}

	while(menuActive==TRUE)
	{

		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX5825 Octal DAC\r\n\r\n");
				if(uchVoltageReference2500mV==TRUE)
				{
					printf("Voltage Reference = 2.500V\r\n");
				}
				else
				{
					//printf("DAC 12-bit (0..4095) setting is %d decimal (%d mV)\r\n",valueDAC,((2048 * valueDAC) / 4096));
					printf("Voltage Reference = 2.048V\r\n");
				}
				printf("Active Channel (0-7, 8=all) is %d\r\n",uchActiveChannel);

				printf("DAC    (hex): ");
				for(i=0;i<8;i++)
					printf("%03x ",(nDAC_shadow[i] & 0x0FFF));
				printf("\r\n");

				printf("CODE   (hex): ");
				for(i=0;i<8;i++)
					printf("%03x ",(nCODE_shadow[i] & 0x0FFF));
				printf("\r\n");

				printf("RETURN (hex): ");
				for(i=0;i<8;i++)
					printf("%03x ",(nRETURN_shadow[i] & 0x0FFF));
				printf("\r\n");

				menu_print_line();
				printf("1.  Set the active channel (0-7, 8=all)\r\n");
				printf("2.  Direct enter the CODE value for channel %d\r\n",uchActiveChannel);
				printf("3.  Load the DAC register from the CODE value for channel %d\r\n",uchActiveChannel);
				printf("4.  Direct enter the RETURN value for channel %d\r\n",uchActiveChannel);
				printf("5.  Send a SW_CLEAR, thereby setting DACs to RETURN value\r\n");
				printf("6.  Send a SW_RESET\r\n");
				printf("7.  ~15Hz Triangle Wave (for ~20 seconds)\r\n");
				printf("8.  Toggle the voltage reference\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;
			case 11:
				uchActiveChannel = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,4,0);
				if(uchActiveChannel>8)
					uchActiveChannel=8;
				nMenuState=0;
				break;
			case 12:
				unCodeRegisterSetting = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,12,0);
				max_MAX5825_set_CODE_register(g_unActivePeripheralAddressI2C,uchActiveChannel,unCodeRegisterSetting);
				menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unCodeRegisterSetting);
				nMenuState = 0;
				break;
			case 13:
				max_MAX5825_LOAD_DAC_from_CODE_register(g_unActivePeripheralAddressI2C,uchActiveChannel);
				menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
				nMenuState=0;
				break;
			case 14:
				unReturnRegisterSetting = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,12,0);
				max_MAX5825_set_RETURN_register(g_unActivePeripheralAddressI2C,uchActiveChannel,unReturnRegisterSetting);
				menu_update_shadow_variable(&nRETURN_shadow[0],uchActiveChannel,unReturnRegisterSetting);
				nMenuState=0;
				break;
			case 15:
				max_MAX5825_send_sw_clear(g_unActivePeripheralAddressI2C);
				for(i=0;i<8;i++)  // clear the shadow DAC & CODE registers (but not RETURN)
				{
					nDAC_shadow[i]=0;
					nCODE_shadow[i]=0;
				}
				nMenuState=0;
				break;
			case 16:
				max_MAX5825_send_sw_reset(g_unActivePeripheralAddressI2C);
				nMenuState=0;
				break;
			case 17:
				printf("Triangle Wave started\r\n");
				for(i=0;i<300;i++)
				{
					for(j=0;j<4096;j=j+100)
					{
						unValueDac = (unsigned int)j;
						max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C,uchActiveChannel,unValueDac);
						menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unValueDac);
						menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
					}
					for(j=4095;j>=0;j=j-100)
					{
						unValueDac = (unsigned int)j;
						max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C,uchActiveChannel,unValueDac);
						menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unValueDac);
						menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
					}
				}
				nMenuState = 0;
				break;
			case 18:
				if(uchVoltageReference2500mV==TRUE)
				{
					if(max_MAX5825_set_internal_reference(g_unActivePeripheralAddressI2C,MAX5825_2048_MV_REFERENCE))
						uchVoltageReference2500mV=FALSE;
				}
				else
				{
					if(max_MAX5825_set_internal_reference(g_unActivePeripheralAddressI2C,MAX5825_2500_MV_REFERENCE))
						uchVoltageReference2500mV=TRUE;
				}
				nMenuState=0;
				break;
			case 19:
				menuActive=FALSE;
				break;
			case KEYPRESS_ARROW_UP:
				if(unValueDac >=4094)
					unValueDac = 4095;
				else
					unValueDac +=1;
				max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C,uchActiveChannel,unValueDac);
				menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unValueDac);
				menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
				nMenuState=0;
				break;
			case KEYPRESS_ARROW_DOWN:
				if(unValueDac<=1)
					unValueDac=0;
				else
					unValueDac -=1;
				max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C,uchActiveChannel,unValueDac);
				menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unValueDac);
				menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
				nMenuState=0;
				break;
			case KEYPRESS_ARROW_LEFT:
				if(unValueDac<=100)
					unValueDac=0;
				else
					unValueDac -=100;
				max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C,uchActiveChannel,unValueDac);
				menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unValueDac);
				menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_RIGHT:
				if(unValueDac >=3995)
					unValueDac = 4095;
				else
					unValueDac += 100;
				max_MAX5825_set_DAC_12_bit_value(g_unActivePeripheralAddressI2C,uchActiveChannel,unValueDac);
				menu_update_shadow_variable(&nCODE_shadow[0],uchActiveChannel,unValueDac);
				menu_copy_shadow_variable_CODE_to_DAC(&nCODE_shadow[0], &nDAC_shadow[0], uchActiveChannel);
				nMenuState = 0;
				break;

			default:
				nMenuState = 0;
				break;

		}
	}
}



void menu_MAX5216()
/**
* \brief       Menu display and processing for MAX5216
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	unsigned int unValueDAC=0;
	int menuActive=TRUE;
	int i,j;

	// Set the active port type to GPIO (the 5216 uses bit-banged GPIO for the SPI-compatible interface)
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_GPIO);

	// Set DACs to zero
	max_MAX5216_set_output_voltage(g_pActiveGPIOPort,0);
	delay(ABOUT_ONE_SECOND/10);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX5216 High-performance, buffered Single DAC\r\n\r\n");
				printf("Current 16-bit DAC setting = %d decimal (%d mV)\r\n",unValueDAC,((unValueDAC * 2500) / 65535));
				menu_print_line();

				printf("1.  Direct enter a value (and hit return)\r\n");
				printf("2.  ~15Hz Triangle Wave (for ~20 seconds)\r\n");
				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				unValueDAC = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,16,0);
				max_MAX5216_set_output_voltage(g_pActiveGPIOPort,unValueDAC);
				nMenuState = 0;
				break;

			case 12:
				printf("Triangle Wave started\r\n");
				fflush(stdout);
				for(i=0;i<300;i++)
				{
					for(j=0;j<65535;j=j+2048)
					{
						max_MAX5216_set_output_voltage(g_pActiveGPIOPort,j);
					}
					for(j=65535;j>=0;j=j-2048)
					{
						max_MAX5216_set_output_voltage(g_pActiveGPIOPort,j);
					}
				}
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_DOWN:
				if(unValueDAC<=100)
					unValueDAC=0;
				else
					unValueDAC -=100;
				max_MAX5216_set_output_voltage(g_pActiveGPIOPort,unValueDAC);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_UP:

				if(unValueDAC >=65435)
					unValueDAC = 65535;
				else
					unValueDAC += 100;
				max_MAX5216_set_output_voltage(g_pActiveGPIOPort,unValueDAC);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_LEFT:
				if(unValueDAC<=1000)
					unValueDAC=0;
				else
					unValueDAC -=1000;
				max_MAX5216_set_output_voltage(g_pActiveGPIOPort,unValueDAC);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_RIGHT:
				if(unValueDAC >=64535)
					unValueDAC = 65535;
				else
					unValueDAC += 1000;
				max_MAX5216_set_output_voltage(g_pActiveGPIOPort,unValueDAC);
				nMenuState = 0;
				break;

			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;

		}
	}
}



void menu_MAX5487()
/**
* \brief       Menu display and processing for MAX5487
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	u8 uchValuePotentiometerA=0;
	u8 uchValuePotentiometerB=0;
	int menuActive=TRUE;

	// Set the active port type to SPI
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_SPI);

	// Set Wiper A & B to Zero
	max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,TRUE,TRUE);
	delay(ABOUT_ONE_SECOND/10);


	while(menuActive==TRUE)
	{

		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX5487 Dual, 256-tap SPI Digital Potentiometer\r\n\r\n");
				printf("8-bit Register/Wiper A value = %d decimal\r\n",uchValuePotentiometerA);
				printf("8-bit Register/Wiper B value = %d decimal\r\n",uchValuePotentiometerB);
				menu_print_line();

				printf("1.  Direct enter a value for Wiper A (and hit return)\r\n");
				printf("2.  Direct enter a value for Wiper B (and hit return)\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;
			case 11:
				uchValuePotentiometerA = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,8,0);
				max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,TRUE,FALSE);
				nMenuState = 0;
				break;
			case 12:
				uchValuePotentiometerB = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,8,0);
				max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,FALSE,TRUE);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_DOWN:
				if(uchValuePotentiometerA<=0)
					uchValuePotentiometerA=0;
				else
					uchValuePotentiometerA -=1;
				if(uchValuePotentiometerB<=0)
					uchValuePotentiometerB=0;
				else
					uchValuePotentiometerB -=1;
				max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,TRUE,TRUE);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_UP:
				if(uchValuePotentiometerA >=254)
					uchValuePotentiometerA = 255;
				else
					uchValuePotentiometerA += 1;
				if(uchValuePotentiometerB >=254)
					uchValuePotentiometerB = 255;
				else
					uchValuePotentiometerB += 1;
				max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,TRUE,TRUE);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_LEFT:
				if(uchValuePotentiometerA<=10)
					uchValuePotentiometerA=0;
				else
					uchValuePotentiometerA -=10;
				if(uchValuePotentiometerB<=10)
					uchValuePotentiometerB=0;
				else
					uchValuePotentiometerB -=10;
				max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,TRUE,TRUE);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_RIGHT:
				if(uchValuePotentiometerA >=245)
					uchValuePotentiometerA = 255;
				else
					uchValuePotentiometerA += 10;
				if(uchValuePotentiometerB >=245)
					uchValuePotentiometerB = 255;
				else
					uchValuePotentiometerB += 10;
				max_MAX5487_change_wiper(g_unActivePeripheralAddressSPI, uchValuePotentiometerA,uchValuePotentiometerB,TRUE,TRUE);
				nMenuState = 0;
				break;

			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;
		}
	}
}


void menu_MAX3232()
/**
* \brief       Menu display and processing for MAX3232
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int i=0;
	u8 uchAsciiCounter = 48; // ASCII number '0'
	u8 uchSessionActive=FALSE;

	int menuActive=TRUE;

	// Set the active port type to UART
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_UART);

	while(menuActive==TRUE)
	{

		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX3232 RS-232 Transceiver (9600,n,8,1)\r\n");
				menu_print_line();
				printf("1.  Full-duplex terminal (Tx and Rx characters are displayed)\r\n");
				printf("2.  Test Loopback of 10 (random) characters (short the DB9's pins 2 and 3)\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11: // Full duplex Rx
				uchSessionActive=TRUE;
				printf("Full-Duplex Test.  Type characters here or in the remote terminal\r\n");
				printf("Hit the <End> key to finish session\r\n");
				menu_print_line();
				while(uchSessionActive==TRUE)
				{
					// Attempt to receive a byte from the RS-232 port for (1) second
					if(receive_byte_with_timeout(g_unActivePeripheralAddressUart,1,&uchInput))
					{
						XUartLite_SendByte(DEFAULT_HYPERTERMINAL_UART_ADDRESS,uchInput);
						if(uchInput==13)  // send a line feed too
						{
							XUartLite_SendByte(DEFAULT_HYPERTERMINAL_UART_ADDRESS,10);
						}
					}
					else
					{
						// Attempt to receive a keypress from the host PC
						if(receive_byte_with_timeout(DEFAULT_HYPERTERMINAL_UART_ADDRESS,1,&uchInput))
						{
							if(uchInput != (KEYPRESS_END))
							{
								XUartLite_SendByte(DEFAULT_HYPERTERMINAL_UART_ADDRESS,uchInput);
								XUartLite_SendByte(g_unActivePeripheralAddressUart,uchInput);
								if(uchInput==13)  // send a line feed too
								{
									XUartLite_SendByte(DEFAULT_HYPERTERMINAL_UART_ADDRESS,10);
									XUartLite_SendByte(g_unActivePeripheralAddressUart,10);
								}
							}
							else
								uchSessionActive=FALSE;
						}
					}
				}
				nMenuState = 0;
				break;

				case 12:
				printf("\r\n");
				for(i=0;i<10;i++)
				{
					XUartLite_SendByte(g_unActivePeripheralAddressUart,uchAsciiCounter);
					// Wait up to 1/10th second to receive data, then fail
					if(receive_byte_with_timeout(g_unActivePeripheralAddressUart,5,&uchInput))
					{
						if(uchInput==uchAsciiCounter)
							printf("Success:  Sent {%c}, Received {%c}\r\n",uchAsciiCounter,uchInput);
						else
							printf("Failed :  Sent {%c}, Received {%c}\r\n",uchAsciiCounter,uchInput);
					}
					else
					{
						i=10;
						printf("Failed :  Timeout waiting for Rx RS-232 character\r\n");
					}
					delay(ABOUT_ONE_SECOND/2);

					uchAsciiCounter++;
					if(uchAsciiCounter > 90)
						uchAsciiCounter=48;
				}
				delay(ABOUT_ONE_SECOND * 3);
				nMenuState=0;
				break;

			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;
		}
	}
}

void menu_MAX14840()
/**
* \brief       Menu display and processing for MAX14840
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int menuActive=TRUE;
	unsigned int unTempValue;

	// Set the active port type to GPIO
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_GPIO);

	// Set RO to be an input, all others to be outputs
	unTempValue = RO;
	XGpio_SetDataDirection(g_pActiveGPIOPort, 1, unTempValue);

	while(menuActive==TRUE)
	{

		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX14840 RS-485 Transceiver\r\n");

				menu_print_line();

				printf("1.  Send a differential (0)\r\n");
				printf("2.  Send a differential (1)\r\n");
				printf("3.  Read differential value\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				// Set the DUT to Tx a zero, and RUT to Rx
				printf("Sending Differential (0)\r\n");
				unTempValue = RE_B | DE;
				XGpio_DiscreteWrite(g_pActiveGPIOPort,1,unTempValue);
				delay(ABOUT_ONE_SECOND);
				nMenuState = 0;
				break;

			case 12:
				// Set the DUT to Tx a zero, and RUT to Rx
				printf("Sending Differential (1)\r\n");
				unTempValue = RE_B | DE | DI;
				XGpio_DiscreteWrite(g_pActiveGPIOPort,1,unTempValue);
				delay(ABOUT_ONE_SECOND);
				nMenuState = 0;
				break;

			case 13:
				// Receive a value
				unTempValue = 0;	// RE_B active, DE in-active
				XGpio_DiscreteWrite(g_pActiveGPIOPort,1,unTempValue);
				unTempValue = XGpio_DiscreteRead(g_pActiveGPIOPort, 1);
				if((unTempValue & RO)==RO)
					printf("Received a (1)\r\n");
				else
					printf("Received a (0)\r\n");
				delay(ABOUT_ONE_SECOND);
				nMenuState = 0;
				break;

			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;
		}
	}
}


void menu_MAX9611_print_statistics(u32 unPeripheralAddressI2C)
/**
* \brief       Print the readings for MAX9611/MAX5380 via Hyperterminal.
*
* \param[in]   unPeripheralAddressI2C  - address of the I2C peripheral in MicroBlaze memory map
*
* \retval      None
*/
{
	unsigned int unTempValue=0;
	printf("\r\n***** Voltage readings from 9611 ADC *****\r\n");

	if(max_MAX9611_read_ADC_voltage(unPeripheralAddressI2C, MAX9611_RS_REGISTER, &unTempValue)==TRUE)
		printf("RS+ (wrt to gnd)                  = %d mV\r\n",((unTempValue * 57300)/4096));
	else
		printf("FAILED TO READ.  RS+ (wrt to gnd).\r\n");
	delay(50000);

	if(max_MAX9611_read_ADC_voltage(unPeripheralAddressI2C,MAX9611_SET_REGISTER, &unTempValue)==TRUE)
		printf("MAX5380 Set/TripPoint Voltage     = %d mV\r\n",((unTempValue * 1100) / 4096));
	else
		printf("FAILED TO READ.  Max5380 set-trip voltage from Max9611\r\n");

	delay(50000);
	if(max_MAX9611_read_ADC_voltage(unPeripheralAddressI2C,MAX9611_CSA_REGISTER, &unTempValue)==TRUE)
	{
		// The ADC inside the 9611 has a default range setting of 440mV spread across the 12 bit range
		// Therefore the voltage measured (in mV) is ADCValue * 440 / 4096
		printf("Voltage across Sense Resistor     = %d mV\r\n",((unTempValue * 440)/4096));

		// The MAX9611 has a fixed 2.5x voltage gain from the CSA input before the op-amp/comparator circuit
		// This is the voltage that is compared to set point voltage from the MAX5380
		printf("2.5x voltage (input to Op-amp)    = %d mV\r\n",((unTempValue * 440 * 5)/(4096 * 2)));

		// Measured load current = Rs (
		printf("Measured load current (Vrs/0.2ohm)= %d mA\r\n",((unTempValue *2200)/4096));
	}
	else
		printf("FAILED TO READ.  Voltage across Sense Resistor.\r\n");

	delay(50000);
	if(max_MAX9611_read_ADC_voltage(unPeripheralAddressI2C,MAX9611_OUT_REGISTER, &unTempValue)==TRUE)
		printf("Op-amp Output                     = %d mV\r\n",((unTempValue * 57300)/4096));
	else
		printf("FAILED TO READ.  Op-amp Output register\r\n");

	delay(50000);
	//printf("*******************************************\r\n");
}


void menu_MAX9611()
/**
* \brief       Menu display and processing for MAX9611
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int i=0;
	u8 uchMax5380Setting = 2;
	int menuActive=TRUE;

	// Set the active port type to I2C
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_I2C);

	if(check_if_i2c_device_is_present(g_nActivePMODPort)==FALSE)
		delay(ABOUT_ONE_SECOND*3);
	// Set voltage on MAX5380, which is used as the current limiting trip-point
	max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
	max_MAX9611_turn_on_2ms_updates(g_unActivePeripheralAddressI2C);
	delay(ABOUT_ONE_SECOND/10);

	while(menuActive==TRUE)
	{

		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX9611 / MAX5380 current limiting PMOD\r\n\r\n");
				menu_MAX9611_print_statistics(g_unActivePeripheralAddressI2C);
				menu_print_line();

				printf("1.  Decrease DAC voltage (decrease current Set/Trip-point)\r\n");
				printf("2.  Increase DAC voltage (increase current Set/Trip-point)\r\n");
				printf("3.  Directly enter DAC voltage\r\n");
				printf("4.  Update 1 time (voltage readings)\r\n");
				printf("5.  Update 10 times\r\n");
				printf("6.  Update 100 times\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				if(uchMax5380Setting<=1)
					uchMax5380Setting=0;
				else
					uchMax5380Setting -= 1;
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;

			case 12:
				if(uchMax5380Setting>=255)
					uchMax5380Setting=255;
				else
					uchMax5380Setting += 1;
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;
			case 13:
				uchMax5380Setting = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,8,0);
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;

			case 14:
				nMenuState = 0;
				break;

			case 15:
				for(i=0;i<10;i++)
				{	menu_cls();
					menu_MAX9611_print_statistics(g_unActivePeripheralAddressI2C);
					delay(ABOUT_ONE_SECOND);
				}
				nMenuState = 0;
				break;

			case 16:
				for(i=0;i<100;i++)
				{
					menu_cls();
					menu_MAX9611_print_statistics(g_unActivePeripheralAddressI2C);
					delay(ABOUT_ONE_SECOND/2);
				}
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_DOWN:
				if(uchMax5380Setting<=1)
					uchMax5380Setting=0;
				else
					uchMax5380Setting -= 1;
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_UP:
				if(uchMax5380Setting>=255)
					uchMax5380Setting=255;
				else
					uchMax5380Setting += 1;
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_LEFT:
				if(uchMax5380Setting<=10)
					uchMax5380Setting=0;
				else
					uchMax5380Setting -= 10;
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_RIGHT:
				if(uchMax5380Setting>=245)
					uchMax5380Setting=255;
				else
					uchMax5380Setting += 10;
				max_MAX5380_change_voltage(g_unActivePeripheralAddressI2C,uchMax5380Setting);
				nMenuState = 0;
				break;

			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;
		}
	}
}


void menu_MAX11205()
/**
* \brief       Menu display and processing for MAX11205
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int nAdcValue=0;
	int menuActive=TRUE;
	u8 uchUseCalibrationMode=FALSE;
	int unNumberSamplesToCapture=1;
	int i;

	// Set the active port type to GPIO
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_GPIO);

	while(menuActive==TRUE)
	{

		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("11025 Single-Channel, Ultra-Low Power, Delta-Sigma ADC with 2-wire interface\r\n\r\n");
				nAdcValue = max_MAX11205_GPIO_SPI_read(g_pActiveGPIOPort,uchUseCalibrationMode);
				printf("ADC reads %d decimal.  (%d * 2500mV) divided by 32768 = %d mV\r\n",nAdcValue,nAdcValue,((nAdcValue * 2500)/32768));
				printf("Calibration mode is ");
				if(uchUseCalibrationMode==TRUE)
					printf("active\r\n");
				else
					printf("not active\r\n");
				menu_print_line();

				printf("1.  Update voltage\r\n");
				printf("2.  Toggle Calibration mode\r\n");
				printf("3.  Direct entry of number of samples to capture\r\n");
				printf("4.  Capture samples indefinitely\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
					//unAdcValue = max_MAX11025_GPIO_SPI_read(g_pActiveGPIOPort);
					//printf("ADC reads %d decimal.  (%d * 2500mV) divided by 32768 = %d mV\r\n",unAdcValue,unAdcValue,((unAdcValue * 2500)/32768));
					//delay(ABOUT_ONE_SECOND * 5);
					nMenuState=0;
				break;
			case 12:
				if(uchUseCalibrationMode==TRUE)
					uchUseCalibrationMode=FALSE;
				else
					uchUseCalibrationMode=TRUE;
				nMenuState=0;
				break;
			case 13:
				unNumberSamplesToCapture = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,16,0);
				for(i=0;i<unNumberSamplesToCapture;i++)
				{
					nAdcValue = max_MAX11205_GPIO_SPI_read(g_pActiveGPIOPort,uchUseCalibrationMode);
					printf("%d of %d samples = %d decimal.  (%d mV)\r\n",i+1,unNumberSamplesToCapture,nAdcValue,((nAdcValue * 2500)/32768));
					delay(ABOUT_ONE_SECOND / 10);
				}
				delay(ABOUT_ONE_SECOND * 2);
				nMenuState=0;
				break;

			case 14:
				printf("Hit any key to exit:\r\n");
				menu_print_line();
				fflush(stdout);
				while(XUartLite_IsReceiveEmpty(DEFAULT_HYPERTERMINAL_UART_ADDRESS))
				{
					nAdcValue = max_MAX11205_GPIO_SPI_read(g_pActiveGPIOPort,uchUseCalibrationMode);
					printf("%d decimal (%d mV)\r\n",nAdcValue,((nAdcValue * 2500)/32768));
					delay(ABOUT_ONE_SECOND / 10);
				}
				menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				delay(ABOUT_ONE_SECOND * 2);
				nMenuState = 0;
				break;



			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;
		}
	}
}

void menu_MAX7304()
/**
* \brief       Menu display and processing for MAX7304
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int menuActive=TRUE;
	u8 uchTxBuffer[3];
	int i=0;
	int x1=0;
	int x2=0;
	int x3=0;
	int x4=0;
	unsigned int unDirectionWord=0;
	u8 uchHighByteDirection=0;
	u8 uchLowByteDirection=0;
	u8 uchHighByteReadValue=0;
	u8 uchLowByteReadValue=0;
	unsigned int unOutputWord;
	u8 uchHighByteOutputValue=0;
	u8 uchLowByteOutputValue=0;
	u8 uchDebounceValue=0;
	//u8 uchHighByte=0;
	//u8 uchLowByte=0;

	// The human eye perceives PWM light intensity in a non-linear fashion wrt to PWM duty-cycle
	// This array provides 32 PWM steps up and 32 PWM steps down
	// to approximate a (human eye perceived) roughly linear intensity increase
	u8 uchDepths[64] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,16,19,23,27,32,38,45,54,64,76,91,108,128,152,181,216,255,
									   255,216,181,152,128,108,91,76,64,54,45,39,32,27,23,19,16,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0};

	u8 auchPwmValues[4];

	// Set the active port type to I2C
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_I2C);
	check_if_i2c_device_is_present(g_nActivePMODPort);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX7304 I2C-Interfaced 16-Port, Level-Translating GPIO and LED Driver\r\n");
				printf("   (Jumper JP1 should be left open, this connects GND to AD0)\r\n\r\n");
				max_MAX7304_get_direction(g_unActivePeripheralAddressI2C, &uchHighByteDirection, &uchLowByteDirection);
				max_MAX7304_get_input(g_unActivePeripheralAddressI2C, &uchHighByteReadValue, &uchLowByteReadValue);
				max_MAX7304_get_debounce_value(g_unActivePeripheralAddressI2C, &uchDebounceValue);

				printf("Input Debounce Value = %d\r\n",uchDebounceValue);
				printf("GPIO[15:0] Direction: (1=output,0=input) ");
				for(i=7;i>=0;i--)
				{
					if(((uchHighByteDirection >> i) & 0x01)==0x01)
						printf("1");
					else
						printf("0");
				}
				printf(" ");
				for(i=7;i>=0;i--)
				{
					if(((uchLowByteDirection >> i) & 0x01)==0x01)
						printf("1");
					else
						printf("0");
				}
				printf("\r\n");
				printf("GPIO[15:0] Read Values:                  ");
				for(i=7;i>=0;i--)
				{
					if(((uchHighByteDirection >> i) & 0x01)==0x01)
						printf("x");
					else
					{
						if(((uchHighByteReadValue >> i) & 0x01)==0x01)
							printf("1");
						else
							printf("0");
					}
				}
				printf(" ");
				for(i=7;i>=0;i--)
				{
					if(((uchLowByteDirection >> i) & 0x01)==0x01)
						printf("x");
					else
					{
						if(((uchLowByteReadValue >> i) & 0x01)==0x01)
							printf("1");
						else
							printf("0");
					}
				}
				printf("\r\n");

				printf("GPIO[15:0] Output Values:                ");
				for(i=7;i>=0;i--)
				{
					if(((uchHighByteDirection >> i) & 0x01)==0x00)
						printf("x");
					else
					{
						if(((uchHighByteOutputValue >> i) & 0x01)==0x01)
							printf("1");
						else
							printf("0");
					}
				}
				printf(" ");
				for(i=7;i>=0;i--)
				{
					if(((uchLowByteDirection >> i) & 0x01)==0x00)
						printf("x");
					else
					{
						if(((uchLowByteOutputValue >> i) & 0x01)==0x01)
							printf("1");
						else
							printf("0");
					}
				}
				printf("\r\n");
				menu_print_line();
				printf("1.  Enable the MAX7304\r\n");
				printf("2.  Directly enter the input/output direction as a 16-bit word\r\n");
				printf("3.  Direct enter a output values as a 16-bit word\r\n");
				printf("4.  Configure LEDs (set PWM registers and constant current registers)\r\n");
				printf("5.  Pulse LEDs (need to execute step #4 before this)\r\n");
				printf("6.  Force an update/read\r\n");
				printf("7.  Directly set the input debounce register value\r\n");

				printf("\r\n9.  Return to Main Menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				max_MAX7304_enable(g_unActivePeripheralAddressI2C, TRUE);
				nMenuState = 0;
				break;
			case 12:
				printf("Here\r\n");
				unDirectionWord = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,16,0);
				// Do not allow bits 15..12 to be set.  They need to be configured as inputs, as on the
				// MAX7304 peripheral module, these pins are DIRECTLY connected to the chip via an LED.
				// Setting them to outputs with a value of zero will let a LARGE amount of current flow
				// through the LEDs.
				unDirectionWord &= 0x0FFF;

				// Assign the direction words
				uchLowByteDirection = (unDirectionWord & 0x00FF);
				unDirectionWord &= 0xFF00;
				unDirectionWord = unDirectionWord >> 8;
				uchHighByteDirection = unDirectionWord;

				max_MAX7304_set_direction(g_unActivePeripheralAddressI2C, uchHighByteDirection, uchLowByteDirection);

				nMenuState = 0;
				break;
			case 13:
				unOutputWord = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,16,0);
				unOutputWord &= 0x0FFF;  // same reason as in case 12.

				// Assign the direction words
				uchLowByteOutputValue = (unOutputWord & 0x00FF);
				unOutputWord &= 0xFF00;
				unOutputWord = unOutputWord >> 8;
				uchHighByteOutputValue = unOutputWord;

				max_MAX7304_set_output(g_unActivePeripheralAddressI2C, uchHighByteOutputValue, uchLowByteOutputValue);
				nMenuState = 0;
				break;
			case 14:
				max_MAX7304_configure_PWM(g_unActivePeripheralAddressI2C,5,5,200,5);
				nMenuState = 0;
				break;
			case 15:
				x1=0;
				x2=0;
				x3=0;
				x4=0;
				// Toggle GPIO
				for(i=0;i<112;i++)
				{
					if(x1<=62)
						x1++;
					if(x1>=16 && x2<=62)
						x2++;
					if(x2>=16 && x3<=62)
						x3++;
					if(x3>=16 && x4<=62)
						x4++;
					auchPwmValues[0] =uchDepths[x1];
					auchPwmValues[1] =uchDepths[x2];
					auchPwmValues[2] =uchDepths[x3];
					auchPwmValues[3] =uchDepths[x4];

					uchTxBuffer[0] = 0x50;  // LED Port12 PWM ratio
					uchTxBuffer[1] = auchPwmValues[0];    // Decimal value is ratio of x / 256ths brightness
					XIic_Send(g_unActivePeripheralAddressI2C, MAX7304_IIC_ADDRESS,
							(u8 *)&uchTxBuffer, 2, XIIC_STOP);
					delay(1000);

					uchTxBuffer[0] = 0x51;  // LED Port12 PWM ratio
					uchTxBuffer[1] = auchPwmValues[1];    // Decimal value is ratio of x / 256ths brightness
					XIic_Send(g_unActivePeripheralAddressI2C, MAX7304_IIC_ADDRESS,
							(u8 *)&uchTxBuffer, 2, XIIC_STOP);
					delay(1000);

					uchTxBuffer[0] = 0x52;  // LED Port12 PWM ratio
					uchTxBuffer[1] = auchPwmValues[2];    // Decimal value is ratio of x / 256ths brightness
					XIic_Send(g_unActivePeripheralAddressI2C, MAX7304_IIC_ADDRESS,
							(u8 *)&uchTxBuffer, 2, XIIC_STOP);
					delay(1000);

					uchTxBuffer[0] = 0x53;  // LED Port12 PWM ratio
					uchTxBuffer[1] = auchPwmValues[3];    // Decimal value is ratio of x / 256ths brightness
					XIic_Send(g_unActivePeripheralAddressI2C, MAX7304_IIC_ADDRESS,
							(u8 *)&uchTxBuffer, 2, XIIC_STOP);
					delay(1000);

					delay(ABOUT_ONE_SECOND/40);
				}
				nMenuState = 0;
				break;
			case 16:	// null - just update the screen with latest stats
				nMenuState = 0;
				break;
			case 17:	// set the input debounce time
				uchDebounceValue = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,5,0);
				max_MAX7304_set_debounce_value(g_unActivePeripheralAddressI2C, uchDebounceValue);
				nMenuState=0;
				break;
			case 19:
				menuActive=FALSE;
				break;

			default:
				nMenuState = 0;
				break;
		}
	}
}

void menu_MAX44000()
/**
* \brief       Menu display and processing for MAX44000
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	unsigned int unLuxReading=0;
	unsigned int unProxReading=0;
	u8 uchLedCurrentSetting=3;
	u8 uchAlsActiveProxNotActive=TRUE;
	int i=0;
	int menuActive=TRUE;

	// Set the active port type to I2C
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_I2C);

	// Configure the MAX44000 if it is present
	if(check_if_i2c_device_is_present(g_nActivePMODPort))
	{
		max_MAX44000_set_led_drive_current(g_unActivePeripheralAddressI2C, uchLedCurrentSetting);
		max_MAX44000_set_als_mode(g_unActivePeripheralAddressI2C);
	}

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX44000 Ambient Light and Proximity Sensor:\r\n\r\n");
				printf("Last lux reading = %d\r\n",unLuxReading);
				printf("Last prox reading = %d\r\n",unProxReading);
				printf("LED Drive current setting = %d\r\n",uchLedCurrentSetting);
				print_seven_segment_number(((float)unLuxReading));
				menu_print_line();

				printf("1.  Retrieve ( 1)    lux reading\r\n");
				printf("2.  Retrieve (20)    lux reading(s)\r\n");
				printf("3.  Retrieve lux indefinitely\r\n");
				printf("4.  Retrieve ( 1)    prox reading\r\n");
				printf("5.  Retrieve (20)    prox reading(s)\r\n");
				printf("6.  Retrieve prox indefinitely\r\n");
				printf("7.  Directly set the LED drive current bits [3:0]\r\n");
				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				if(uchAlsActiveProxNotActive==FALSE)
				{
					if(max_MAX44000_set_als_mode(g_unActivePeripheralAddressI2C))
						uchAlsActiveProxNotActive=TRUE;
					delay(ABOUT_ONE_SECOND / 20);
				}
				if(max_MAX44000_readLux(g_unActivePeripheralAddressI2C, &unLuxReading))
				{
					printf("Lux Value = %d\r\n",unLuxReading);
					print_seven_segment_number(((float)unLuxReading));
				}
				else
					printf("FAILED to retrieve Lux value\r\n");
				delay(ABOUT_ONE_SECOND * 1);
				nMenuState = 0;
				break;

			case 12:
				if(uchAlsActiveProxNotActive==FALSE)
				{
					if(max_MAX44000_set_als_mode(g_unActivePeripheralAddressI2C))
						uchAlsActiveProxNotActive=TRUE;
					delay(ABOUT_ONE_SECOND / 20);
				}
				for(i=0;i<20;i++)
				{
					if(max_MAX44000_readLux(g_unActivePeripheralAddressI2C, &unLuxReading))
					{
						printf("Lux Value %d of 20 = %d\r\n",i,unLuxReading);
						print_seven_segment_number(((float)unLuxReading));
					}
					else
						printf("FAILED to retrieve Lux value\r\n");

					delay(ABOUT_ONE_SECOND / 3);
				}
				nMenuState = 0;
				break;

			case 13:
				if(uchAlsActiveProxNotActive==FALSE)
				{
					if(max_MAX44000_set_als_mode(g_unActivePeripheralAddressI2C))
						uchAlsActiveProxNotActive=TRUE;
					delay(ABOUT_ONE_SECOND / 20);
				}
				printf("Hit any key to exit:\r\n");
				menu_print_line();
				fflush(stdout);
				while(XUartLite_IsReceiveEmpty(DEFAULT_HYPERTERMINAL_UART_ADDRESS))
				{
					if(max_MAX44000_readLux(g_unActivePeripheralAddressI2C, &unLuxReading))
					{
						printf("Lux Value = %d\r\n",unLuxReading);
						print_seven_segment_number(((float)unLuxReading));
					}
					else
						printf("FAILED to retrieve Lux value\r\n");
					delay(ABOUT_ONE_SECOND / 2);
				}
				menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = 0;
				break;
			case 14:
				if(uchAlsActiveProxNotActive==TRUE)
				{
					if(max_MAX44000_set_prox_mode(g_unActivePeripheralAddressI2C))
						uchAlsActiveProxNotActive=FALSE;
					delay(ABOUT_ONE_SECOND / 20);
				}
				if(max_MAX44000_readProx(g_unActivePeripheralAddressI2C, &unProxReading))
				{
					printf("Prox Value = %d\r\n",unProxReading);
					print_seven_segment_number(((float)unProxReading));
				}
				else
					printf("FAILED to retrieve Prox value\r\n");
				delay(ABOUT_ONE_SECOND * 1);
				nMenuState = 0;
				break;

			case 15:
				if(uchAlsActiveProxNotActive==TRUE)
				{
					if(max_MAX44000_set_prox_mode(g_unActivePeripheralAddressI2C))
						uchAlsActiveProxNotActive=FALSE;
					delay(ABOUT_ONE_SECOND / 20);
				}
				for(i=0;i<20;i++)
				{
					if(max_MAX44000_readProx(g_unActivePeripheralAddressI2C, &unProxReading))
					{
						printf("Prox Value %d of 20 = %d\r\n",i,unProxReading);
						print_seven_segment_number(((float)unProxReading));
					}
					else
						printf("FAILED to retrieve Prox value\r\n");
					delay(ABOUT_ONE_SECOND / 2);
				}
				nMenuState = 0;
				break;

			case 16:
				if(uchAlsActiveProxNotActive==TRUE)
				{
					if(max_MAX44000_set_prox_mode(g_unActivePeripheralAddressI2C))
						uchAlsActiveProxNotActive=FALSE;
					delay(ABOUT_ONE_SECOND / 20);
				}
				printf("Hit any key to exit:\r\n");
				menu_print_line();
				fflush(stdout);
				while(XUartLite_IsReceiveEmpty(DEFAULT_HYPERTERMINAL_UART_ADDRESS))
				{
					if(max_MAX44000_readProx(g_unActivePeripheralAddressI2C, &unProxReading))
					{
						printf("Prox Value = %d\r\n",unProxReading);
						print_seven_segment_number(((float)unProxReading));
					}
					else
						printf("FAILED to retrieve Prox value\r\n");
					delay(ABOUT_ONE_SECOND / 2);
				}
				menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = 0;
				break;
			case 17:
				uchLedCurrentSetting = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,4,0);
				max_MAX44000_set_led_drive_current(g_unActivePeripheralAddressI2C, uchLedCurrentSetting);
				nMenuState=0;
				break;
			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;

		}
	}
}

void menu_MAX4824()
/**
* \brief       Menu display and processing for MAX4824
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	u8 uchChannelToActuate=0;
	int nPowerSaveActive=FALSE;
	int menuActive=TRUE;

	// Set the active port type to GPIO
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_GPIO);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX4824 Octal Relay Driver:\r\n\r\n");
				if(nPowerSaveActive==TRUE)
					printf("Powersave mode active\r\n");
				else
					printf("Powersave mode in-active\r\n");
				print_seven_segment_number(0.0f);
				menu_print_line();

				printf("1.  Direct enter a channel to activate    (0-7) (and hit return)\r\n");
				printf("2.  Direct enter a channel to de-activate (0-7) (and hit return)\r\n");
				printf("3.  Set all outputs active    (SETB   pin)\r\n");
				printf("4.  Set all outputs in-active (RESETB pin)\r\n");
				printf("5.  Toggle Powersave mode     (PSAVE  pin)\r\n");
				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				uchChannelToActuate = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,3,0);
				max_MAX4824_set_output_active(g_pActiveGPIOPort,uchChannelToActuate,nPowerSaveActive);
				nMenuState = 0;
				break;

			case 12:
				uchChannelToActuate = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,3,0);
				max_MAX4824_set_output_inactive(g_pActiveGPIOPort,uchChannelToActuate,nPowerSaveActive);
				nMenuState = 0;
				break;
			case 13:
				max_MAX4824_set_all_active(g_pActiveGPIOPort, FALSE);
				nMenuState = 0;
				break;
			case 14:
				max_MAX4824_set_all_inactive(g_pActiveGPIOPort,nPowerSaveActive);
				nMenuState = 0;
				break;
			case 15:
				if(nPowerSaveActive==FALSE)
					nPowerSaveActive=TRUE;
				else
					nPowerSaveActive=FALSE;
				max_MAX4824_set_powersave(g_pActiveGPIOPort, nPowerSaveActive);
				nMenuState=0;
				break;

			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;

		}
	}
}

void menu_print_day_of_week(u8 uchDayOfWeek)
/**
* \brief       Print the day of the week.
*
* \param[in]   uchDayOfWeek      - index of DOW (0=Sunday, 1=Monday, ... , 6=Saturday)
*
* \retval      None
*/
{
	switch(uchDayOfWeek)
	{
		case 0:
			printf("Sunday");
			break;
		case 1:
			printf("Monday");
			break;
		case 2:
			printf("Tuesday");
			break;
		case 3:
			printf("Wednesday");
			break;
		case 4:
			printf("Thursday");
			break;
		case 5:
			printf("Friday");
			break;
		case 6:
			printf("Saturday");
			break;
		default:
			printf("Unknown Day");
			break;
	}
}


void menu_DS3231M()
/**
* \brief       Menu display and processing for DS3231M
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	u8 uch32KhzEnabled=TRUE;
	int menuActive=TRUE;
	struct maximDateTime t;
	float fTemp=0.0f;

	t.uchHour = 0;
	t.uchMinute = 0;
	t.uchSecond = 0;
	t.uchMonth = 1;
	t.uchDate = 1;
	t.uchYear = 1;
	t.uchDow = 0;

	// Set the active port type to I2C
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_I2C);

	if(check_if_i2c_device_is_present(g_nActivePMODPort)==TRUE)
		max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("DS3231M I2C Real-Time Clock:\r\n\r\n");
				printf("Use arrow keys to set time. (L/R = Hours, U/D = Minutes)\r\n");
				max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);
				max_DS3231M_get_temp(g_unActivePeripheralAddressI2C, &fTemp);
				printf("Last time reading = %02d:%02d:%02d\r\n",t.uchHour,t.uchMinute,t.uchSecond);
				printf("Last date reading = ");
				menu_print_day_of_week(t.uchDow);
				printf(" %d/%d/%d\r\n",t.uchMonth,t.uchDate,(t.uchYear+2000));
				print_seven_segment_time((struct maximDateTime *)&t);
				printf("Last temp reading = ");
				printf_temp(fTemp, TRUE, TRUE);
				if(uch32KhzEnabled==TRUE)
					printf("32kHz is enabled\r\n");
				else
					printf("32kHz is not enabled\r\n");
				menu_print_line();

				printf("1.  Update  time reading\r\n");
				printf("2.  Update  temp reading\r\n");
				printf("3.  Toggle enable/disable of 32kHz\r\n");
				printf("4.  Increment Day of Week (ex: Mon->Tues)\r\n");
				printf("5.  Increment Month       (ex: Mar->Apr )\r\n");
				printf("6.  Increment Date        (ex: 3rd->4th )\r\n");
				printf("7.  Increment Year        (ex: 2012->13 )\r\n");
				printf("8.  Decrement Year        (ex: 2012->11 )\r\n");
				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);
				nMenuState = 0;
				break;

			case 12:
				max_DS3231M_get_temp(g_unActivePeripheralAddressI2C, &fTemp);
				nMenuState = 0;
				break;
			case 13:
				if(uch32KhzEnabled==TRUE)
					uch32KhzEnabled=FALSE;
				else
					uch32KhzEnabled=TRUE;
				max_DS3231M_32kHz(g_unActivePeripheralAddressI2C, uch32KhzEnabled);
				nMenuState = 0;
				break;
			case 14:
				t.uchDow++;
				if(t.uchDow>=7)
					t.uchDow=0;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case 15:
				t.uchMonth++;
				if(t.uchMonth>=12)
					t.uchMonth=0;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case 16:
				t.uchDate++;
				if(t.uchDate>=31)
					t.uchDate=0;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case 17:
				t.uchYear++;
				if(t.uchYear>=200)
					t.uchYear=0;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case 18:
				t.uchYear--;
				if(t.uchYear==0)
					t.uchYear=199;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;

			case KEYPRESS_ARROW_UP:
				max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);
				t.uchMinute += 1;
				if(t.uchMinute > 59)
				{
					t.uchMinute = 0;
					t.uchHour += 1;
					if(t.uchHour>=24)
						t.uchHour = 0;
				}
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case KEYPRESS_ARROW_DOWN:
				max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);
				if(t.uchMinute==0)
				{
					t.uchMinute = 59;
					if(t.uchHour==0)
						t.uchHour = 23;
					else
						t.uchHour -=1;
				}
				else
					t.uchMinute -=1 ;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case KEYPRESS_ARROW_RIGHT:  // increase the hour
				max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);
				t.uchHour += 1;
				if(t.uchHour>=24)
						t.uchHour = 0;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case KEYPRESS_ARROW_LEFT:	// decrease the hour
				max_DS3231M_get_time(g_unActivePeripheralAddressI2C, (struct maximDateTime *)&t);
				if(t.uchHour==0)
					t.uchHour = 23;
				else
					t.uchHour -=1;
				max_DS3231M_set_time(g_unActivePeripheralAddressI2C,(struct maximDateTime *)&t);
				nMenuState = 0;
				break;
			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;

		}
	}
}


void menu_MAX31723()
/**
* \brief       Menu display and processing for MAX31723
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int menuActive=TRUE;
	float fTemp=0.0f;
	int displayCelsius=TRUE;
	int i;
	float fHighAlarmSetting=0.0f;
	float fLowAlarmSetting=0.0f;

	// Set the active port type to SPI
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_SPI);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX31723 Digital Thermometer:\r\n\r\n");
				max_MAX31723_get_temp(&fLowAlarmSetting, g_unActivePeripheralAddressSPI, MAX31723_LOW_ALARM_READ);
				max_MAX31723_get_temp(&fHighAlarmSetting, g_unActivePeripheralAddressSPI, MAX31723_HIGH_ALARM_READ);
				printf("Last temp reading = ");
				printf_temp(fTemp,displayCelsius,TRUE);
				printf("Low Alarm Setpoint = ");
				printf_temp(fLowAlarmSetting,displayCelsius,TRUE);
				printf("High Alarm Setpoint = ");
				printf_temp(fHighAlarmSetting,displayCelsius,TRUE);
				menu_print_line();

				printf("1.  Retrieve ( 1)    temp reading\r\n");
				printf("2.  Retrieve (20)    temp reading(s)\r\n");
				printf("3.  Retrieve temp indefinitely\r\n");
				printf("4.  Directly enter low alarm setpoint\r\n");
				printf("5.  Directly enter high alarm setpoint\r\n");

				printf("6.  Toggle display degC / degF\r\n");
				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				max_MAX31723_get_temp(&fTemp, g_unActivePeripheralAddressSPI,MAX31723_TEMP_READ);
				print_seven_segment_temperature(fTemp,displayCelsius);
				nMenuState = 0;
				break;
			case 12:
				printf("\r\n");
				fflush(stdout);
				for(i=0;i<20;i++)
				{
					max_MAX31723_get_temp(&fTemp, g_unActivePeripheralAddressSPI,MAX31723_TEMP_READ);
					printf("%d of 20 samples = ",i+1);
					printf_temp(fTemp,displayCelsius,TRUE);
					print_seven_segment_temperature(fTemp,displayCelsius);
					delay(ABOUT_ONE_SECOND / 2);
				}
				nMenuState=0;
				break;
			case 13:
				printf("Hit any key to exit:\r\n");
				menu_print_line();
				fflush(stdout);
				while(XUartLite_IsReceiveEmpty(DEFAULT_HYPERTERMINAL_UART_ADDRESS))
				{
					max_MAX31723_get_temp(&fTemp, g_unActivePeripheralAddressSPI,MAX31723_TEMP_READ);
					printf_temp(fTemp,displayCelsius,TRUE);
					print_seven_segment_temperature(fTemp,displayCelsius);
					delay(ABOUT_ONE_SECOND/2);
				}
				menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState=0;
				break;
			case 14:
				fLowAlarmSetting = (float)menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,7,0);
				max_MAX31723_set_alarm(fLowAlarmSetting,g_unActivePeripheralAddressSPI,MAX31723_LOW_ALARM_WRITE);
				nMenuState=0;
				break;
			case 15:
				fHighAlarmSetting = (float)menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,7,0);
				max_MAX31723_set_alarm(fHighAlarmSetting,g_unActivePeripheralAddressSPI,MAX31723_HIGH_ALARM_WRITE);
				nMenuState=0;
				break;
			case 16:
				if(displayCelsius==TRUE)
					displayCelsius=FALSE;
				else
					displayCelsius=TRUE;
				print_seven_segment_temperature(fTemp,displayCelsius);
				nMenuState = 0;
				break;
			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;

		}
	}
}


void menu_DS1086L()
/**
* \brief       Menu display and processing for DS1086L
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int menuActive=TRUE;
	unsigned int unDacValue=0;
	unsigned int unPrescalarValue=0;
	unsigned int unJitterValue=0;
	int nOffsetDelta=0;

	// Set the active port type to I2C
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_I2C);

	if(check_if_i2c_device_is_present(g_nActivePMODPort))
		max_DS1086L_setup(g_unActivePeripheralAddressI2C, g_pActiveGPIOPort);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("DS1086L Spread Spectrum EconOscillator:\r\n\r\n");
				max_DS1086_get_dac(g_unActivePeripheralAddressI2C, &unDacValue);
				max_DS1086_get_prescalar(g_unActivePeripheralAddressI2C, &unPrescalarValue);
				max_DS1086_get_jitter(g_unActivePeripheralAddressI2C, &unJitterValue);
				max_DS1086_get_offset_delta(g_unActivePeripheralAddressI2C,&nOffsetDelta);
				printf("Last DAC reading = %d (decimal)\r\n",unDacValue);
				printf("Last Jitter Value = %d (decimal)\r\n",unJitterValue);
				printf("Last Prescalar Value = %d (decimal)\r\n",unPrescalarValue);
				printf("Last Offset Delta Value = %d (decimal)\r\n",nOffsetDelta);

				menu_print_line();
				printf("1.  Directly Set DAC (for VCO) (0..1023)\r\n");
				printf("2.  Directly Set Prescalar which divides by 2^n where n=(0..8)\r\n");
				printf("3.  Directly Set Jitter bits JS[4:0] (see datasheet)\r\n");
				printf("4.  Increase the Offset Delta (value must be -6 to +6)\r\n");
				printf("5.  Decrease the Offset Delta (value must be -6 to +6)\r\n");

				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				unDacValue = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,10,0);
				max_DS1086_set_dac(g_unActivePeripheralAddressI2C,unDacValue);
				nMenuState = 0;
				break;

			case 12:
				unPrescalarValue = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,4,0);
				if(unPrescalarValue >=8)		// capped at 8
					unPrescalarValue=8;
				max_DS1086_set_prescalar(g_unActivePeripheralAddressI2C, unPrescalarValue);
				nMenuState = 0;
				break;

			case 13:
				unJitterValue = menu_get_direct_entry(DEFAULT_HYPERTERMINAL_UART_ADDRESS,4,0);
				max_DS1086_set_jitter(g_unActivePeripheralAddressI2C, unJitterValue);
				nMenuState = 0;
				break;

			case 14:
				nOffsetDelta++;
				if(nOffsetDelta>6)
					nOffsetDelta=6;
				max_DS1086_set_offset_delta(g_unActivePeripheralAddressI2C, nOffsetDelta);
				nMenuState = 0;
				break;

			case 15:
				nOffsetDelta--;
				if(nOffsetDelta<-6)
					nOffsetDelta=-6;
				max_DS1086_set_offset_delta(g_unActivePeripheralAddressI2C, nOffsetDelta);
				nMenuState = 0;
				break;
			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;

		}
	}
}




void menu_MAX31855()
/**
* \brief       Menu display and processing for MAX31855
*
* \param       None
*
* \retval      None
*/
{
	int nMenuState = 0;
	u8 uchInput=0;
	int menuActive=TRUE;
	float fInternalTemp=0.0f;
	float fProbeTemp=0.0f;
	int i;
	int displayCelsius=TRUE;

	// Set the active port type to SPI
	max_set_PMOD_port(g_nActivePMODPort, PMOD_PORT_TYPE_SPI);

	while(menuActive==TRUE)
	{
		switch(nMenuState)
		{
			case 0:
				menu_cls();
				printf("MAX31855 Thermocouple to Digital Converter:\r\n\r\n");
				printf("Last internal temp reading = ");
				printf_temp(fInternalTemp,displayCelsius,TRUE);
				printf("Last probe temp reading = ");
				printf_temp(fProbeTemp,displayCelsius,TRUE);
				menu_print_line();

				printf("1.  Retrieve (  1)    temp reading\r\n");
				printf("2.  Retrieve ( 20)    temp reading(s)\r\n");
				printf("3.  Retrieve temp indefinitely (hit any key to exit)\r\n");
				printf("4.  Toggle display degC / degF\r\n");
				printf("\r\n9.  Return to main menu\r\n");
				menu_print_prompt();
				nMenuState = 1;
				break;
			case 1:
				uchInput = menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = uchInput+10;
				break;

			case 11:
				max_MAX31855_read_internalTemp(&fInternalTemp,g_unActivePeripheralAddressSPI);
				max_MAX31855_read_thermoCoupleTemp(&fProbeTemp,g_unActivePeripheralAddressSPI);
				print_seven_segment_temperature(fProbeTemp,displayCelsius);
				delay(ABOUT_ONE_SECOND);
				nMenuState = 0;
				break;
			case 12:
				printf("\r\n");
				for(i=0;i<20;i++)
				{
					max_MAX31855_read_internalTemp(&fInternalTemp,g_unActivePeripheralAddressSPI);
					max_MAX31855_read_thermoCoupleTemp(&fProbeTemp,g_unActivePeripheralAddressSPI);
					printf("{internal temp, probe temp} = {");
					printf_temp(fInternalTemp,displayCelsius,FALSE);
					printf(",");
					printf_temp(fProbeTemp,displayCelsius,FALSE);
					printf("}\r\n");
					print_seven_segment_temperature(fProbeTemp,displayCelsius);
					delay(ABOUT_ONE_SECOND);
				}
				nMenuState = 0;
				break;
			case 13:
				while(XUartLite_IsReceiveEmpty(DEFAULT_HYPERTERMINAL_UART_ADDRESS))
				{
					max_MAX31855_read_internalTemp(&fInternalTemp,g_unActivePeripheralAddressSPI);
					max_MAX31855_read_thermoCoupleTemp(&fProbeTemp,g_unActivePeripheralAddressSPI);
					printf("{internal temp, probe temp} = {");
					printf_temp(fInternalTemp,displayCelsius,FALSE);
					printf(",");
					printf_temp(fProbeTemp,displayCelsius,FALSE);
					printf("}\r\n");
					print_seven_segment_temperature(fProbeTemp,displayCelsius);
					delay(ABOUT_ONE_SECOND);
				}
				menu_retrieve_keypress(DEFAULT_HYPERTERMINAL_UART_ADDRESS);
				nMenuState = 0;
				break;
			case 14:
				if(displayCelsius==TRUE)
					displayCelsius=FALSE;
				else
					displayCelsius=TRUE;
				print_seven_segment_temperature(fProbeTemp,displayCelsius);
				nMenuState = 0;
				break;
			case 19:
				menuActive=FALSE;
				break;
			default:
				nMenuState = 0;
				break;
		}
	}
}

